package org.insa.graphs.algorithm.shortestpath;


import java.util.Arrays;

import java.util.Collections;
import java.util.List;
import java.util.ArrayList;

import org.insa.graphs.algorithm.AbstractInputData.Mode;
import org.insa.graphs.algorithm.AbstractSolution.Status;
import org.insa.graphs.algorithm.utils.BinaryHeap;
import org.insa.graphs.algorithm.utils.ElementNotFoundException;
import org.insa.graphs.model.Arc;
import org.insa.graphs.model.Graph;
import org.insa.graphs.model.Node;
import org.insa.graphs.model.Path;
public class DijkstraAlgorithm extends ShortestPathAlgorithm {
	
	
	protected void init(Label[] tabLabel, ShortestPathData data)
	{
		
		//Initialization 
        for(Node n:data.getGraph().getNodes())
        {
        	tabLabel[n.getId()]=new Label(n);
        }
        //tabLabel[data.getOrigin().getId()].setCout(0);
		
	}
	
	
    public DijkstraAlgorithm(ShortestPathData data) {
        super(data);
    }

    @Override
    protected ShortestPathSolution doRun() {
    	double temps_debut=System.currentTimeMillis();
        final ShortestPathData data = getInputData();
        ShortestPathSolution solution = null;
        Graph graph = data.getGraph();
        final int nbNodes=graph.size();
        BinaryHeap<Label> tas=new BinaryHeap<Label>();
        double coutPrec=0.0; //Sert uniquement pour les tests
        Label []tabLabel=new Label[nbNodes];
        init(tabLabel, data);
        //Initialization 
        /*for(Node n:graph.getNodes())
        {
        	tabLabel[n.getId()]=new Label(n);
        }*/
        tabLabel[data.getOrigin().getId()].setCout(0);
        tas.insert(tabLabel[data.getOrigin().getId()]);
        this.notifyOriginProcessed(data.getOrigin());
        int nbExplore=0;// sert uniquement pour les tests
        int nbMarque=1;// sert uniquement pour les tests
        int iterations=0; // sert uniquement pour les tests
        while(!tabLabel[data.getDestination().getId()].isMarque())
        {
        	iterations++; //sert uniquement pour les tests.
        	Node X;
        	try
        	{
        	X=tas.findMin().getSommetCourant();
        	}
        	catch(org.insa.graphs.algorithm.utils.EmptyPriorityQueueException e)
        	{
        		System.out.println("Tas vide \n");
        		break;
        	}
        	tabLabel[X.getId()].setMarque(true);
        	this.notifyNodeMarked(X);
        	if(tabLabel[X.getId()].getCoutTotal()<coutPrec)
        	{
        		System.out.println("Problème! Les coûts ne sont pas croissants \n");
        		System.out.println("Coût: "+tabLabel[X.getId()].getCoutTotal()+"\n");
        		System.out.println("Coût précédent: "+ coutPrec +"\n");
        	}
        	coutPrec=tabLabel[X.getId()].getCoutTotal();
        	if(tabLabel[data.getDestination().getId()].isMarque())
        	{
        		break;
        	}
        	nbMarque++;
        	try
			{
				tas.remove(tabLabel[X.getId()]);
			}
			catch(ElementNotFoundException e) {System.out.println("The element was not found in the binary heap \n");}
        	int nbSuccesseurs=0;
        	for(Arc a: X.getSuccessors())
        	{
        		nbSuccesseurs++;
        		Node Y=a.getDestination();
        		if(!tabLabel[Y.getId()].isMarque())
        		{
        			//double cout_avant=tabLabel[Y.getId()].getCout();
        			if(((tabLabel[X.getId()].getCout()+data.getCost(a))<tabLabel[Y.getId()].getCout()) && data.isAllowed(a))
        			{		tabLabel[Y.getId()].setCout(tabLabel[X.getId()].getCout()+data.getCost(a));
        					try
        					{
        						tas.remove(tabLabel[Y.getId()]);
        						tas.insert(tabLabel[Y.getId()]);
                				tabLabel[Y.getId()].setPere(a);
                				
        						
        					}
        					catch(ElementNotFoundException e) {tas.insert(tabLabel[Y.getId()]);
            				tabLabel[Y.getId()].setPere(a);
            				this.notifyNodeReached(Y);
            				nbExplore++;}
        		
        			}
        			
        		}
        	}
        	if(nbSuccesseurs==X.getNumberOfSuccessors())
        	{
        		//System.out.println("Nombre de successeurs OK \n");
        	}
        	else
        	{
        		System.out.println("PROBLEME: Nombre de successeurs NOT OK ! ! ! ! \n");
        	}
        	
        }
     // Destination has no predecessor, the solution is infeasible...
       if (tabLabel[data.getDestination().getId()].getPere() == null) {
            solution = new ShortestPathSolution(data, Status.INFEASIBLE);
        }
        else {
        this.notifyDestinationReached(data.getDestination());
        ArrayList<Arc> arcs = new ArrayList<>();
        Arc arc = tabLabel[data.getDestination().getId()].getPere();
        while (arc != null) {
            arcs.add(arc);
            arc = tabLabel[arc.getOrigin().getId()].getPere();
        }
        System.out.println("Nombre d'arcs: "+arcs.size() + "\n");
        System.out.println("Nombre d'itérations: "+iterations + "\n");
        System.out.println("Nombre de noeuds marqués: "+ nbMarque + "\n");
        System.out.println("Nombre de noeuds explorés: "+ nbExplore + "\n");
        // Reverse the path...
        Collections.reverse(arcs);
        
        //Vérification grâce à Path.createShortestPathFromNodes
        Path p= new Path(graph,arcs);
        //List<Node> noeuds=new ArrayList<Node>();
        
        if((int) p.getLength()*100== (int) tabLabel[data.getDestination().getId()].getCout()*100)
        {
        	System.out.println("La longueur du plus court chemin est la même que celle calculée grâce à l'algo \n");
        }
        else
        {
        	
        	System.out.println("ERREUR: La longueur du plus court chemin n'est la même que celle calculée grâce à l'algo \n");
        	System.out.println("Cout pcc="+ tabLabel[data.getDestination().getId()].getCout());
        	System.out.println("Cout algo="+p.getLength());
        }
        /*noeuds.add(data.getDestination());
        
        if(data.getMode()==Mode.LENGTH)
        {
        	Path shortestpath=Path.createShortestPathFromNodes(graph, noeuds);
            
            if(shortestpath.getLength()==p.getLength())
            {
            	System.out.println("La longueur du plus court chemin est la même que celle calculée grâce à Dijkstra \n");
            }
            else
            {
            	System.out.println("ERREUR: La longueur du plus court chemin n'est la même que celle calculée grâce à Dijkstra \n");
            }
        }
        
        else
        {
        	Path fastestpath=Path.createFastestPathFromNodes(graph, noeuds);
            
            if(fastestpath.getMinimumTravelTime()==p.getMinimumTravelTime())
            {
            	System.out.println("La longueur du plus court chemin est la même que celle calculée grâce à Dijkstra \n");
            }
            else
            {
            	System.out.println("ERREUR: La longueur du plus court chemin n'est la même que celle calculée grâce à Dijkstra \n");
            }
        }*/
        
        //Vérification de la validation du path
        if(p.isValid())
        {
        	System.out.println("Félicitation votre path est valide \n");
        }
        else
        {
        	System.out.println("Erreur! votre path est invalide \n");
        }
        // Create the final solution.
        solution = new ShortestPathSolution(data, Status.OPTIMAL, new Path(graph, arcs));
        }
       double temps_fin=System.currentTimeMillis();
       double total_temps=temps_fin-temps_debut;
       System.out.println("Duree d'execution:"+total_temps);
       return solution;
    }

}
