import org.insa.graphs.model.*;
import org.insa.graphs.algorithm.*;
import org.insa.graphs.algorithm.shortestpath.*;
import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertTrue;

import java.io.BufferedInputStream;
import java.io.DataInputStream;
import java.io.FileInputStream;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.Collections;
import org.insa.graphs.model.Arc;
import org.insa.graphs.model.Graph;
import org.insa.graphs.model.Node;
import org.insa.graphs.model.Point;
import org.insa.graphs.model.Path;
import org.insa.graphs.model.RoadInformation;
import org.insa.graphs.model.RoadInformation.RoadType;
import org.insa.graphs.model.io.BinaryGraphReader;
import org.insa.graphs.model.io.GraphReader;
import org.junit.BeforeClass;
import org.junit.Test;
import org.junit.Assume;
import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.junit.runners.Parameterized;
import org.junit.runners.Parameterized.Parameter;
import org.junit.runners.Parameterized.Parameters;

@RunWith(Parameterized.class)
public class OptiTest {
	private static Graph graph1;
	private static Graph graph2;
	private static Graph graph3;
	
	
    
    @Parameters
    public static Collection<ShortestPathData> data() {
    	//**************************************************************
    	//**************************************************************
    	//CREATION DU GRAPHE DE LA CARTE HAUTE-GARONNE
        final String mapName1 = "C:\\Users\\nmouk\\Documents\\GIT\\MAPS\\haute-garonne\\europe\\france\\haute-garonne.mapgr" ;
        try {
        	final FileInputStream F1= new FileInputStream(mapName1);
        }
        catch(Exception e) {System.out.println("Fichier non existant!");}
        // Create a graph reader.
        try {
        final GraphReader reader = new BinaryGraphReader(
                new DataInputStream(new BufferedInputStream(new FileInputStream(mapName1))));
        
        
        // Small graph use for tests
        graph1 = reader.read();
        }
        catch(Exception e) {System.out.println("Fichier non lu!");}
      //******************************************************************
      //******************************************************************
      //CREATION DU GRAPHE DE LA CARTE CARRE
        final String mapName2 = "C:\\Users\\nmouk\\Documents\\GIT\\MAPS\\carre\\extras\\carre.mapgr" ;
       
        try {
        	final FileInputStream F2= new FileInputStream(mapName2);
        }
        catch(Exception e) {System.out.println("Fichier non existant!");}
        // Create a graph reader.
        try {
        final GraphReader reader = new BinaryGraphReader(
                new DataInputStream(new BufferedInputStream(new FileInputStream(mapName2))));
        
        
        // Small graph use for tests
        graph2 = reader.read();
        }
        catch(Exception e) {System.out.println("Fichier non lu!");}
        
        //******************************************************************
        //******************************************************************
        //CREATION DU GRAPHE DE LA CARTE POLYNESIE FRANCAISE
          final String mapName3 = "C:\\Users\\nmouk\\Documents\\GIT\\MAPS\\french-polynesia\\oceania\\french-polynesia.mapgr" ;
         
          try {
          	final FileInputStream F3= new FileInputStream(mapName2);
          }
          catch(Exception e) {System.out.println("Fichier non existant!");}
          // Create a graph reader.
          try {
          final GraphReader reader = new BinaryGraphReader(
                  new DataInputStream(new BufferedInputStream(new FileInputStream(mapName3))));
          
          
          // Small graph use for tests
          graph3 = reader.read();
          }
          catch(Exception e) {System.out.println("Fichier non lu!");}
          
          
        
    	Collection<ShortestPathData> objects = new ArrayList<>();
    	
    	//***************CARTE HAUTE-GARONNE****************************
    	//**************************************************************
    	//CHEMIN NUL AVEC 2 MODES DE DEPLACEMENTS
    	objects.add(new ShortestPathData(graph1, graph1.get(10991),graph1.get(10991),ArcInspectorFactory.getAllFilters().get(0)));
    	objects.add(new ShortestPathData(graph1, graph1.get(10991),graph1.get(10991),ArcInspectorFactory.getAllFilters().get(2)));
    	//CHEMIN AVEC 3 MODES DE DEPLACEMENTS
    	objects.add(new ShortestPathData(graph1, graph1.get(10991),graph1.get(10991),ArcInspectorFactory.getAllFilters().get(0)));
    	objects.add(new ShortestPathData(graph1, graph1.get(10991),graph1.get(10991),ArcInspectorFactory.getAllFilters().get(2)));
    	//***************CARTE CARRE************************************
    	//**************************************************************
    	//CHEMIN NUL AVEC 2 MODES DE DEPLACEMENTS
    	objects.add(new ShortestPathData(graph2, graph2.get(19),graph2.get(19),ArcInspectorFactory.getAllFilters().get(0)));
    	objects.add(new ShortestPathData(graph2, graph2.get(19),graph2.get(19),ArcInspectorFactory.getAllFilters().get(2)));
    	//CHEMIN AVEC 2 MODES DE DEPLACEMENTS
    	objects.add(new ShortestPathData(graph2, graph2.get(19),graph2.get(5),ArcInspectorFactory.getAllFilters().get(0)));
    	objects.add(new ShortestPathData(graph2, graph2.get(19),graph2.get(5),ArcInspectorFactory.getAllFilters().get(2)));
    	objects.add(new ShortestPathData(graph2, graph2.get(19),graph2.get(5),ArcInspectorFactory.getAllFilters().get(1)));
    	
    	
    	
    	//*************CARTE POLYNESIE FRANCAISE**********************
    	//************************************************************
    	//CHEMIN IMPOSSIBLE AVEC 2 MODES DE DEPLACEMENTS
    	objects.add(new ShortestPathData(graph3, graph3.get(8302),graph3.get(4904),ArcInspectorFactory.getAllFilters().get(0)));
    	objects.add(new ShortestPathData(graph3, graph3.get(8302),graph3.get(4904),ArcInspectorFactory.getAllFilters().get(2)));
    return objects;
    }
    
    
    @Parameter
    public ShortestPathData parameter;
    
    
    @Test
    public void testCheminNul()
    {	//Shortest path, all roads allowed
    	Assume.assumeTrue(parameter.getOrigin().getId()==parameter.getDestination().getId() && parameter.getGraph().getMapId() != graph3.getMapId());
    	ShortestPathAlgorithm Dijkstra= new DijkstraAlgorithm(parameter);
    	ShortestPathAlgorithm BF= new BellmanFordAlgorithm(parameter);
    	DijkstraAlgorithm Astar= new AStarAlgorithm(parameter);
    	assertEquals(Dijkstra.run().isFeasible(),BF.run().isFeasible());
    	assertEquals(Astar.run().isFeasible(),BF.run().isFeasible());
    }
    
   
    @Test
    public void testCheminCourt()
    {
    	Assume.assumeTrue(parameter.getOrigin().getId()!=parameter.getDestination().getId() && parameter.getGraph().getMapId() != graph3.getMapId());
    	ShortestPathAlgorithm Dijkstra= new DijkstraAlgorithm(parameter);
    	ShortestPathAlgorithm BF= new BellmanFordAlgorithm(parameter);
    	DijkstraAlgorithm Astar= new AStarAlgorithm(parameter);
    	assertEquals(Dijkstra.run().getPath().getLength(), BF.run().getPath().getLength(),0.01);
    	assertEquals(Astar.run().getPath().getLength(),BF.run().getPath().getLength(),0.01);
    	assertEquals(Dijkstra.run().getPath().getMinimumTravelTime(), BF.run().getPath().getMinimumTravelTime(),0.01);
    	assertEquals(Astar.run().getPath().getMinimumTravelTime(),BF.run().getPath().getMinimumTravelTime(),0.01);
    	
    }
    
    @Test
    public void testCheminImpossible()
    {	//Shortest path, all roads allowed
    	Assume.assumeTrue(parameter.getGraph().getMapId() == graph3.getMapId());
    	ShortestPathAlgorithm Dijkstra= new DijkstraAlgorithm(parameter);
    	ShortestPathAlgorithm BF= new BellmanFordAlgorithm(parameter);
    	DijkstraAlgorithm Astar= new AStarAlgorithm(parameter);
    	assertEquals(Dijkstra.run().isFeasible(),BF.run().isFeasible());
    	assertEquals(Astar.run().isFeasible(),BF.run().isFeasible());
    }
   
    /*
    @Test
    public void testCheminLong()
    {
    	ShortestPathData data1=new ShortestPathData(graph, longPath.getOrigin(),longPath.getDestination(),ArcInspectorFactory.getAllFilters().get(0));
    	ShortestPathAlgorithm Dijkstra= new DijkstraAlgorithm(data1);
    	ShortestPathAlgorithm BF= new BellmanFordAlgorithm(data1);
    	DijkstraAlgorithm Astar= new AStarAlgorithm(data1);
    	assertEquals(Dijkstra.run().toString(),BF.run().toString());
    	assertEquals(Astar.run().toString(),BF.run().toString());
    }
    
    @Test
    public void testgrandeBoucle()
    {
    	ShortestPathData data1=new ShortestPathData(graph, loopPath.getOrigin(),loopPath.getDestination(),ArcInspectorFactory.getAllFilters().get(0));
    	ShortestPathAlgorithm Dijkstra= new DijkstraAlgorithm(data1);
    	ShortestPathAlgorithm BF= new BellmanFordAlgorithm(data1);
    	DijkstraAlgorithm Astar= new AStarAlgorithm(data1);
    	assertEquals(Dijkstra.run().toString(),BF.run().toString());
    	assertEquals(Astar.run().toString(),BF.run().toString());
    }
    
    @Test
    public void testBoucle()
    {
    	ShortestPathData data1=new ShortestPathData(graph, longLoopPath.getOrigin(),longLoopPath.getDestination(),ArcInspectorFactory.getAllFilters().get(0));
    	ShortestPathAlgorithm Dijkstra= new DijkstraAlgorithm(data1);
    	ShortestPathAlgorithm BF= new BellmanFordAlgorithm(data1);
    	DijkstraAlgorithm Astar= new AStarAlgorithm(data1);
    	assertEquals(Dijkstra.run().toString(),BF.run().toString());
    	assertEquals(Astar.run().toString(),BF.run().toString());
    }
    
    @Test
    public void Testinvalide()
    {
    	ShortestPathData data1=new ShortestPathData(graph, nodes[0],nodes[5],ArcInspectorFactory.getAllFilters().get(0));
    	ShortestPathAlgorithm Dijkstra= new DijkstraAlgorithm(data1);
    	ShortestPathAlgorithm BF= new BellmanFordAlgorithm(data1);
    	DijkstraAlgorithm Astar= new AStarAlgorithm(data1);
    	assertEquals(Dijkstra.run().toString(),BF.run().toString());
    	assertEquals(Astar.run().toString(),BF.run().toString());
    }
    
    */
    
}
